/**********************************************************************
  PROJECT: Text Demo No 1
  Source: tex2_demo.cpp
  Chapter reference: 4
***********************************************************************
  Description: 
    Display text on the client area
  Topics:
    1. Displaying text using TextOut()
	2. Reformatting text to fit client area
***********************************************************************/


#include <windows.h>  // Standard Windows header

// Public string for text display demonstration 
char TextMsg[] = {"Visual Studio 97 provides the development "
 "environment in which your programming and Web site "
 "development packages run. This integrated set of tools runs "
 "under Windows 95 or Windows NT. With Developer Studio, "
 "you can create, test, and refine your applications "
 "and Web sites all in one place. Now in Visual Studio 97, "
 "you can work with a greater variety of documents without "
 "leaving the development environment. Of course, you can "
 "create source files and some other documents by using "
 "Developer Studio products directly. In the Developer Studio "
 "environment, you can also create and edit documents (ActiveX "
 " documents) associated with any ActiveX components you "
 "have installed, such as word processing programs or "
 "spreadsheet programs." };


//************************
//       WinMain()
//************************
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

// Registering the wmdclass structure
     RegisterClassEx (&wndclass) ;

	 hwnd = CreateWindow (szAppName,
		  "Text Display Demo No. 1",        // Window caption
           WS_OVERLAPPEDWINDOW,				// style
		   CW_USEDEFAULT,					// initial x
		   CW_USEDEFAULT,					// initial y
		   CW_USEDEFAULT,					// initial x size
		   CW_USEDEFAULT,					// initial y size
		   NULL,							// parent window handle
		   NULL,							// window menu handle
		   hInstance,						// program handle
		   NULL) ;							// creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

//************************
//    Message loop
//************************
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }


//************************
//    Windows Procedure
//************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{

 	 static int  cxChar, cxCaps, cyChar ;// Character dimensions
	 static int  cxClient, cyClient;     // Client area parameters
     HDC         hdc ;                   // handle to device context
	 int j;                              // Offset into string
	 int i;                              // characters since last space
	 char *startptr, *endptr;            // String pointers
	 int cyScreen;                       // Screen row holder

	 // Structures
     PAINTSTRUCT ps;
	 TEXTMETRIC  tm;
	 SIZE        textsize;				 // Test string size
   
	 switch (iMsg)
          {

	      case WM_CREATE :
               hdc = GetDC (hwnd) ;
               GetTextMetrics (hdc, &tm) ;

               // Calculate and store character dimensions
               cxChar = tm.tmAveCharWidth ;
               cxCaps = ((tm.tmPitchAndFamily & 1) ? 3 : 2) * cxChar / 2 ;
               cyChar = tm.tmHeight + tm.tmExternalLeading ;

               ReleaseDC (hwnd, hdc) ;
               return 0 ;

          case WM_SIZE:
			  // Determine and store size of client area
			  cxClient = LOWORD(lParam);
			  cyClient = HIWORD(lParam);
			  return 0;


          case WM_PAINT :
               hdc = BeginPaint (hwnd, &ps) ;

               // Initialize variables
			   cyScreen = cyChar;      // screen row counter
			   startptr = TextMsg;     // start position pointer
			   endptr = TextMsg;       // end position pointer
			   j = 0;                  // length of string
			   i = 0;                  // characters since last space

			   // Screen text line display loop
			   while(*startptr) {
				   if(*startptr == 0x20){    // if character is space
					   GetTextExtentPoint32 (hdc, endptr, j, &textsize);

                   // ASSERT: 
                   //       textsize.cx is the current length of the string
				   //       cxClient is the abcsisa of the client area
                   //       (both in logical units)
                   // INVARIANT:
                   //        i = characters since last space
				   //        j = length of current string
				   // startptr = pointer to sub-string start
				   //   endptr = pointer to sub-string end
				   // Test for line overflow condition. If so, adjust
				   // sub-string to preceeding space and display	    
  					   if(cxClient - (2 * cxChar) < textsize.cx) {
                          j = j - i;
						  startptr = startptr - i;
					      TextOut (hdc, cxChar, cyScreen, endptr, j);
						  cyScreen = cyScreen + cyChar;
						  endptr = startptr;
						  j = 0;
						}
					   // End of space character processing. 
                       // Reset chars-to-previous-space counter, whether
					   // or not string was displayed
					   i = 0;
				   }
                // End of processing for any text character
                // Update sub-string pointer and counters
                startptr++;
				j++;
				i++;
			   }
			   // End of while loop
			   // Display last text sub-string
			   j = j - i;
			   TextOut (hdc, cxChar, cyScreen, endptr, j);

               EndPaint (hwnd, &ps);

               return 0 ;


          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }
