/**********************************************************************
  PROJECT: Text Demo No 2
  Source: tex2_demo.cpp
  Chapter reference: 4
***********************************************************************
  Description: 
    Display text on the client area
  Topics:
    1. Displaying text using DrawText()
	2. Using DrawText() to reformat text to fit client area 
***********************************************************************/

#include <windows.h>  // Standard Windows header

// Public string for text display demonstration 
char TextStr[] = {"Visual Studio 97 provides the development "
 "environment in which your programming and Web site "
 "development packages run. This integrated set of tools runs "
 "under Windows 95 or Windows NT. With Developer Studio, "
 "you can create, test, and refine your applications "
 "and Web sites all in one place.\n"
 "Now in Visual Studio 97, you can work with a greater "
 "variety of documents without leaving the development "
 "environment. Of course, you can create source files and some "
 "other documents by using Developer Studio products "
 "directly.\n"
 "In the Developer Studio environment, you can also create and "
 "edit documents (ActiveX documents) associated with any "
 "ActiveX components you have installed, such as word "
 "processing programs or spreadsheet programs." };


//************************
//       WinMain()
//************************
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

// Registering the wmdclass structure
     RegisterClassEx (&wndclass) ;

	 hwnd = CreateWindow (szAppName,
		  "Text Display Demo No.2",         // Window caption
           WS_OVERLAPPEDWINDOW,				// style
		   CW_USEDEFAULT,					// initial x
		   CW_USEDEFAULT,					// initial y
		   CW_USEDEFAULT,					// initial x size
		   CW_USEDEFAULT,					// initial y size
		   NULL,							// parent window handle
		   NULL,							// window menu handle
		   hInstance,						// program handle
		   NULL) ;							// creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

//************************
//    Message loop
//************************
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }


//************************
//    Windows Procedure
//************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{

 	 static int  cxChar, cyChar ;        // Character dimensions
	 static int  cxClient, cyClient;     // Client area parameters
     HDC         hdc ;                   // handle to device context

	 // Structures
     PAINTSTRUCT ps;
	 TEXTMETRIC  tm;
	 RECT		 textRect;                // RECT-type
   
	 switch (iMsg)
          {

	      case WM_CREATE :
               hdc = GetDC (hwnd) ;
               GetTextMetrics (hdc, &tm) ;

               // Calculate and store character dimensions
               cxChar = tm.tmAveCharWidth ;
               cyChar = tm.tmHeight + tm.tmExternalLeading ;

               ReleaseDC (hwnd, hdc) ;
               return 0 ;

          case WM_SIZE:
			  // Determine and store size of client area
			  cxClient = LOWORD(lParam);
			  cyClient = HIWORD(lParam);
			  return 0;

          case WM_PAINT :
               hdc = BeginPaint (hwnd, &ps) ;

               // Initialize variables
			   SetRect (&textRect,                // address of structure
				        2 * cxChar,               // x for start
						cyChar,                   // y for start
						cxClient -(2 * cxChar),   // x for end
						cyClient);                // y for end 

               // Call display function using left-aligned and wordbreak
			   // controls
               DrawText( hdc, TextStr, -1, &textRect,
				         DT_LEFT | DT_WORDBREAK);

               EndPaint (hwnd, &ps);

               return 0 ;


          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }
