/**********************************************************************
  PROJECT: Text Demo No 3
  Source: tex3_demo.cpp
  Chapter reference: 4
***********************************************************************
  Description: 
    Display text using font selection and paths
  Topics:
    1. Creating a logical fonts
	2. Displaying text using a stroked path
	3. Displaying text using a stroked and filled path
***********************************************************************/

#include <windows.h>  // Standard Windows header

//************************
//       WinMain()
//************************
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demo" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

// Registering the wmdclass structure
     RegisterClassEx (&wndclass) ;

	 hwnd = CreateWindow (szAppName,
		  "Text Display Demo No.3",         // Window caption
           WS_OVERLAPPEDWINDOW,				// style
		   CW_USEDEFAULT,					// initial x
		   CW_USEDEFAULT,					// initial y
		   CW_USEDEFAULT,					// initial x size
		   CW_USEDEFAULT,					// initial y size
		   NULL,							// parent window handle
		   NULL,							// window menu handle
		   hInstance,						// program handle
		   NULL) ;							// creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

//************************
//    Message loop
//************************
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }


//************************
//    Windows Procedure
//************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{

     HDC         hdc ;                   // handle to device context
	 HFONT       hFont;                  // Handle to a font
	 HPEN        aPen;
	 HBRUSH      aBrush;

	 // Structures
     PAINTSTRUCT ps;
   
	 switch (iMsg)
          {

          case WM_PAINT :
               hdc = BeginPaint (hwnd, &ps) ;

			   // Create a logical font
			   hFont = CreateFont (
					   50 * GetDeviceCaps (hdc, LOGPIXELSY) / 72, // height
					   0,                  // width
					   0,                  // escapement angle
					   0,                  // orientation angle
					   FW_NORMAL,          // weight
					   1,                  // italics
					   0,                  // not underlined
					   0,                  // not strikedout
					   DEFAULT_CHARSET,    // character set
					   OUT_DEFAULT_PRECIS, // precision
					   CLIP_DEFAULT_PRECIS,// clipping precision
					   PROOF_QUALITY,      // quality
					   DEFAULT_PITCH | FF_DONTCARE, // pitch and family
					   "Times New Roman");           // typeface name

			   SelectObject( hdc, hFont);  // Select font into DC

               // Start a path for stroked text
			   // Set background mix to TRANSPARENT mode
			   BeginPath (hdc);
			   SetBkMode(hdc, TRANSPARENT);
			   TextOut(hdc, 20, 20, "This Text is STROKED", 20);
			   EndPath(hdc);
			   // Create a custOm black pen, 2 pixels wide
			   aPen = CreatePen(PS_SOLID, 2, 0);
			   SelectObject(hdc, aPen);           // select it into DC
               StrokePath (hdc);                  // Stroke the path

			   // Second path for stroked and filled text
			   BeginPath (hdc);
			   SetBkMode(hdc, TRANSPARENT);
			   TextOut(hdc, 20, 110, "Stroked and Filled", 18);
			   EndPath(hdc);
			   // Get and select a stock pen and brush
			   aPen = GetStockObject(BLACK_PEN);
			   aBrush = GetStockObject(LTGRAY_BRUSH);
			   SelectObject(hdc, aPen);
			   SelectObject(hdc, aBrush);
               StrokeAndFillPath (hdc);          // Stroke and fill path

			   // Clean-up and end paint processing
			   DeleteObject(hFont);
               EndPaint (hwnd, &ps);

               return 0 ;


          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }
