/**********************************************************************
  PROJECT: Caret Demo
  Source: cardemo.cpp
  Chapter reference: 4
***********************************************************************
  Description:
    Caret handling demonstration program 
   Topics:
    1. Using the caret to control text input
	2. Obtaning and releasing the keyboard focus
	3. Moving and manipulating the caret 
***********************************************************************/


#include <windows.h>  // standard Windows header

char TextStr1[] = {"<Bksp> to edit. <Enter> to end line. <Esc> to exit>\n\n"
                    "  name:  \n"
					"  address:  \n" 
					"  city:  \n"
					"  state:  \n"
					"  zip code:  "};

//************************
//       WinMain()
//************************
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "Demos" ;
     HWND        hwnd ;
     MSG         msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

// Registering the wmdclass structure
     RegisterClassEx (&wndclass) ;

	 hwnd = CreateWindow (szAppName,
		  "Caret Demo",                     // Window caption
           WS_OVERLAPPEDWINDOW,				// style
		   CW_USEDEFAULT,					// initial x
		   CW_USEDEFAULT,					// initial y
		   CW_USEDEFAULT,					// initial x size
		   CW_USEDEFAULT,					// initial y size
		   NULL,							// parent window handle
		   NULL,							// window menu handle
		   hInstance,						// program handle
		   NULL) ;							// creation parameters

     ShowWindow (hwnd, iCmdShow) ;
     UpdateWindow (hwnd) ;

//************************
//    Message loop
//************************
     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }


//************************
//    Windows Procedure
//************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam,
						  LPARAM lParam) {

 	 static int  cxChar, cyChar ;        // character dimensions
	 static int  cxClient, cyClient;     // client area parameters
	 static int  xCaret, yCaret;         // caret position
	 static int  xLimit;                 // left limit of line
	 static	int	 formEnd = 0;			 // 1 if Esc key pressed
	 static int  lineNum = 1;            // input line
     static HDC         hdc ;            // handle to private DC
	 char aChar;                         // storage for character code

	 // Structures
     PAINTSTRUCT ps;
	 TEXTMETRIC  tm;
	 RECT		 textRect;                // RECT-type
	 HFONT       hFont;

	 switch (iMsg)
          {

		 case WM_CREATE :
             hdc = GetDC (hwnd) ;


		     // Create a logical font
			 hFont = CreateFont (
			         16 * GetDeviceCaps (hdc, LOGPIXELSY) / 72, // height
				     0,                  // width
				     0,                  // escapement angle
				     0,                  // orientation angle
				     FW_NORMAL,          // weight
				     0,                  // not italics
				     0,                  // not underlined
				     0,                  // not strikedout
				     DEFAULT_CHARSET,    // character set
				     OUT_DEFAULT_PRECIS, // precision
				     CLIP_DEFAULT_PRECIS,// clipping precision
				     PROOF_QUALITY,      // quality
				     DEFAULT_PITCH | FF_DONTCARE, // pitch and family
				     "Courier");         // typeface name

			   SelectObject( hdc, hFont);  // Select font into DC

               GetTextMetrics (hdc, &tm) ;

               // Calculate and store character dimensions
               cxChar = tm.tmAveCharWidth ;
               cyChar = tm.tmHeight + tm.tmExternalLeading ;
			   // Store size of client area
			   cxClient = LOWORD(lParam);
			   cyClient = HIWORD(lParam);
			   // Store initial caret position
			   xCaret = xLimit = 10;
			   yCaret = 3;
               return 0 ;

		 case WM_SIZE:
			   // Determine and store size of client area
			   cxClient = LOWORD(lParam);
			   cyClient = HIWORD(lParam);
			   return 0;
         
		  case WM_PAINT :
               BeginPaint (hwnd, &ps) ;
               // Initialize rectangle structure
			   SetRect (&textRect,                // address of structure
				        2 * cxChar,               // x for start
						cyChar,                   // y for start
						cxClient -(2 * cxChar),   // x for end
						cyClient);                // y for end 
               // Display multi-line text string 
               DrawText( hdc, TextStr1, -1, &textRect,
				         DT_LEFT | DT_WORDBREAK);

               EndPaint (hwnd, &ps);

               return 0 ;
 
          //****************************
          //     keystroke procssing
          //****************************
		  // Character code processing
		  case WM_CHAR:

			   HideCaret(hwnd);
			   aChar = (char) wParam;

			   switch (wParam) {    // wParam holds virtual-key code
				 case '\r':		   // <Enter> key pressed
					yCaret++;
					aChar = 0x20;
					if(yCaret == 4)    // in address: line
					   xCaret = xLimit = 13;
					
					if(yCaret == 5)    // in city: line
					   xCaret = xLimit = 10;
					
					if(yCaret == 6)    // in state: line
					   xCaret = xLimit = 11;
					
					if(yCaret == 7)    // in zip code: line
					   xCaret = xLimit = 14;
					
					if(yCaret > 7) {    // Enter key ignore on last line
					   yCaret--;
					}
					break;

				 case '\b':				// Backspace key pressed
					if (xCaret > xLimit) {
						aChar = 0x20;       // Replace with space
						xCaret--;
						// Display the blank character
						TextOut (hdc, xCaret * cxChar, yCaret * cyChar,
							     &aChar, 1);
					}
					break;

				 case 0x1b:			// Esc key processing
					formEnd = 1;
								  // Destroy the caret
					HideCaret(hwnd);
					DestroyCaret();
					PostQuitMessage (0) ;
                    return 0 ;
					break;

				 default:
					// Display the character if <Esc> not pressed
					if(formEnd == 0) {
						TextOut (hdc, xCaret * cxChar, yCaret * cyChar,
						         &aChar, 1);
						xCaret++;
					}
					break;
			  }
				 if(formEnd == 0) {
						SetCaretPos(xCaret * cxChar, yCaret * cyChar);
						ShowCaret(hwnd);
				}
			  return 0;
		
		  case WM_SETFOCUS:
			   if(formEnd == 0) {
				  CreateCaret (hwnd, NULL, cxChar / 4, cyChar);
				  SetCaretPos(xCaret * cxChar, yCaret * cyChar);
				  ShowCaret(hwnd);
			   }
			   return 0;

		  case WM_KILLFOCUS:
			    // Destroy the caret
			    HideCaret(hwnd);
			    DestroyCaret();
			    return 0;
			  
          case WM_DESTROY :
               PostQuitMessage (0) ;
               return 0 ;
          }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
     }
